@@ -44,7 +44,7 @@
 
 /* Please increase version number with every change 
    (don't forget to update dll.desc) */
-#define DLL_VERSION "1.0.10"
+#define DLL_VERSION "1.0.11"
 
 #ifdef _AIX
 # include "lalloca.h"		/* MUST come first for AIX! */
@@ -99,6 +99,15 @@
 # define PATH_MAX       1024
 #endif
 
+#if defined(HAVE_OS2_H)
+# define DIR_SEP        ";"
+#elif defined(HAVE_WINDOWS_H)
+# define DIR_SEP        ";"
+#else
+# define DIR_SEP        ":"
+#endif
+
+
 #include "sane/sanei_config.h"
 #define DLL_CONFIG_FILE "dll.conf"
 #define DLL_ALIASES_FILE "dll.aliases"
@@ -304,6 +313,14 @@ load (struct backend *be)
 # define PREFIX "libsane-"
 # ifdef __hpux
 #   define POSTFIX ".sl.%u"
+#elif defined (HAVE_WINDOWS_H)
+#   undef PREFIX
+#   define PREFIX "cygsane-"
+#   define POSTFIX "-%u.dll"
+#elif defined (HAVE_OS2_H)
+#   undef PREFIX
+#   define PREFIX ""
+#   define POSTFIX ".dll"
 # else
 #   define POSTFIX ".so.%u"
 # endif
@@ -357,19 +374,25 @@ load (struct backend *be)
     }
   DBG (3, "load: searching backend `%s' in `%s'\n", be->name, src);
 
-  dir = strsep (&src, ":");
+  dir = strsep (&src, DIR_SEP);
 
   while (dir)
     {
+#ifdef HAVE_OS2_H   /* only max 8.3 names possible for dlls on OS/2 */
+      snprintf (libname, sizeof (libname), "%s/" PREFIX "%.2s%.6s" POSTFIX,
+		dir, be->name, strlen(be->name)>8 ? (be->name)+strlen(be->name)-6 :
+                                            (be->name)+2, V_MAJOR);
+#else
       snprintf (libname, sizeof (libname), "%s/" PREFIX "%s" POSTFIX,
 		dir, be->name, V_MAJOR);
+#endif
       DBG (4, "load: trying to load `%s'\n", libname);
       fp = fopen (libname, "r");
       if (fp)
 	break;
       DBG (4, "load: couldn't open `%s' (%s)\n", libname, strerror (errno));
 
-      dir = strsep (&src, ":");
+      dir = strsep (&src, DIR_SEP);
     }
   if (orig_src)
     free (orig_src);
@@ -999,7 +1022,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 
   DBG (3,
        "sane_control_option(handle=%p,option=%d,action=%d,value=%p,info=%p)\n",
-       handle, option, action, value, info);
+       handle, option, action, value, (void *) info);
   return (*(op_ctl_option_t)s->be->op[OP_CTL_OPTION]) (s->handle, option, action, value,
 					     info);
 }
@@ -1009,7 +1032,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
 {
   struct meta_scanner *s = handle;
 
-  DBG (3, "sane_get_parameters(handle=%p,params=%p)\n", handle, params);
+  DBG (3, "sane_get_parameters(handle=%p,params=%p)\n", handle, (void *) params);
   return (*(op_get_params_t)s->be->op[OP_GET_PARAMS]) (s->handle, params);
 }
 
@@ -1029,7 +1052,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data, SANE_Int max_length,
   struct meta_scanner *s = handle;
 
   DBG (3, "sane_read(handle=%p,data=%p,maxlen=%d,lenp=%p)\n",
-       handle, data, max_length, length);
+       handle, data, max_length, (void *) length);
   return (*(op_read_t)s->be->op[OP_READ]) (s->handle, data, max_length, length);
 }
 
@@ -1057,6 +1080,6 @@ sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
   struct meta_scanner *s = handle;
 
-  DBG (3, "sane_get_select_fd(handle=%p,fdp=%p)\n", handle, fd);
+  DBG (3, "sane_get_select_fd(handle=%p,fdp=%p)\n", handle, (void *) fd);
   return (*(op_get_select_fd_t)s->be->op[OP_GET_SELECT_FD]) (s->handle, fd);
 }
