@@ -48,20 +48,20 @@
 
 #include "../include/sane/config.h"
 
-#define BUILD 60
+#define BUILD 66
 #define MAX_DEBUG
-#define WARMUP_TIME 30
+#define WARMUP_TIME 60
 #define CALIBRATION_HEIGHT 2.5
 
-#if 0
-#ifdef HAVE_SYS_SHM_H
+/* Use a reader process if possible (usually faster) */
+#if defined (HAVE_SYS_SHM_H) && (!defined (USE_PTHREAD)) && (!defined (HAVE_OS2_H))
 #define USE_FORK
 #define SHM_BUFFERS 10
 #endif
-#endif
 
 #define TUNE_CALIBRATOR
 
+/* Send coarse white or black calibration to stdout */
 #if 0
 #define SAVE_WHITE_CALIBRATION
 #endif
@@ -69,6 +69,16 @@
 #define SAVE_BLACK_CALIBRATION
 #endif
 
+/* Debug calibration, print total brightness of the scanned image */
+#if 0
+#define DEBUG_BRIGHTNESS
+#endif
+
+/* Debug calibration, print black mark values */
+#if 0
+#define DEBUG_BLACK
+#endif
+
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -114,9 +124,16 @@ SANE_Bool little_endian;
 SANE_Bool debug_options = SANE_FALSE;
 
 static SANE_String_Const mode_list[] = {
-  SANE_I18N ("Color"),
-  SANE_I18N ("Gray"),
-  SANE_I18N ("Lineart"),
+  SANE_VALUE_SCAN_MODE_COLOR,
+  SANE_VALUE_SCAN_MODE_GRAY,
+  SANE_VALUE_SCAN_MODE_LINEART,
+  0
+};
+
+static SANE_String_Const gray_mode_list[] = {
+  GT68XX_COLOR_RED,
+  GT68XX_COLOR_GREEN,
+  GT68XX_COLOR_BLUE,
   0
 };
 
@@ -278,7 +295,7 @@ setup_scan_request (GT68xx_Scanner * s, GT68xx_Scan_Request * scan_request)
   if (s->val[OPT_FULL_SCAN].w == SANE_TRUE)
     {
       scan_request->x0 -= s->dev->model->x_offset;
-      scan_request->y0 -= (s->dev->model->y_offset - s->dev->model->y_offset_calib);
+      scan_request->y0 -= (s->dev->model->y_offset);
       scan_request->xs += s->dev->model->x_offset;
       scan_request->ys += s->dev->model->y_offset;
     }
@@ -293,12 +310,12 @@ setup_scan_request (GT68xx_Scanner * s, GT68xx_Scan_Request * scan_request)
   else
     scan_request->depth = 8;
 
-  if (strcmp (s->val[OPT_MODE].s, "Color") == 0)
+  if (strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_COLOR) == 0)
     scan_request->color = SANE_TRUE;
   else
     scan_request->color = SANE_FALSE;
 
-  if (strcmp (s->val[OPT_MODE].s, "Lineart") == 0)
+  if (strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_LINEART) == 0)
     {
       SANE_Int xs =
 	SANE_UNFIX (scan_request->xs) * scan_request->xdpi / MM_PER_INCH +
@@ -336,7 +353,7 @@ calc_parameters (GT68xx_Scanner * s)
   val = s->val[OPT_MODE].s;
 
   s->params.last_frame = SANE_TRUE;
-  if (strcmp (val, "Gray") == 0 || strcmp (val, "Lineart") == 0)
+  if (strcmp (val, SANE_VALUE_SCAN_MODE_GRAY) == 0 || strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) == 0)
     s->params.format = SANE_FRAME_GRAY;
   else				/* Color */
     s->params.format = SANE_FRAME_RGB;
@@ -353,7 +370,7 @@ calc_parameters (GT68xx_Scanner * s)
       return status;
     }
 
-  if (strcmp (val, "Lineart") == 0)
+  if (strcmp (val, SANE_VALUE_SCAN_MODE_LINEART) == 0)
     s->params.depth = 1;
   else
     s->params.depth = scan_params.depth;
@@ -376,14 +393,6 @@ calc_parameters (GT68xx_Scanner * s)
   if (s->params.format == SANE_FRAME_RGB)
     s->params.bytes_per_line *= 3;
 
-  DBG (4, "calc_parameters: format=%d\n", s->params.format);
-  DBG (4, "calc_parameters: last frame=%d\n", s->params.last_frame);
-  DBG (4, "calc_parameters: lines=%d\n", s->params.lines);
-  DBG (4, "calc_parameters: pixels per line=%d\n", s->params.pixels_per_line);
-  DBG (4, "calc_parameters: bytes per line=%d\n", s->params.bytes_per_line);
-  DBG (4, "calc_parameters: Pixels %dx%dx%d\n",
-       s->params.pixels_per_line, s->params.lines, 1 << s->params.depth);
-
   DBG (5, "calc_parameters: exit\n");
   return status;
 }
@@ -398,7 +407,7 @@ create_bpp_list (GT68xx_Scanner * s, SANE_Int * bpp)
   s->bpp_list[0] = count;
   for (count = 0; bpp[count] != 0; count++)
     {
-      s->bpp_list[count + 1] = bpp[count];
+      s->bpp_list[s->bpp_list[0] - count] = bpp[count];
     }
   return SANE_STATUS_GOOD;
 }
@@ -445,7 +454,18 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_MODE].constraint_type = SANE_CONSTRAINT_STRING_LIST;
   s->opt[OPT_MODE].size = max_string_size (mode_list);
   s->opt[OPT_MODE].constraint.string_list = mode_list;
-  s->val[OPT_MODE].s = strdup ("Gray");
+  s->val[OPT_MODE].s = strdup (SANE_VALUE_SCAN_MODE_GRAY);
+
+  /* scan mode */
+  s->opt[OPT_GRAY_MODE_COLOR].name = "gray-mode-color";
+  s->opt[OPT_GRAY_MODE_COLOR].title = SANE_I18N ("Gray mode color");
+  s->opt[OPT_GRAY_MODE_COLOR].desc = SANE_I18N ("Selects which scan color is used "
+						"gray mode (default: green).");
+  s->opt[OPT_GRAY_MODE_COLOR].type = SANE_TYPE_STRING;
+  s->opt[OPT_GRAY_MODE_COLOR].constraint_type = SANE_CONSTRAINT_STRING_LIST;
+  s->opt[OPT_GRAY_MODE_COLOR].size = max_string_size (gray_mode_list);
+  s->opt[OPT_GRAY_MODE_COLOR].constraint.string_list = gray_mode_list;
+  s->val[OPT_GRAY_MODE_COLOR].s = strdup (GT68XX_COLOR_GREEN);
 
   /* scan source */
   s->opt[OPT_SOURCE].name = SANE_NAME_SCAN_SOURCE;
@@ -470,16 +490,15 @@ init_options (GT68xx_Scanner * s)
   s->val[OPT_PREVIEW].w = SANE_FALSE;
 
   /* lamp on */
-  s->opt[OPT_LAMP_ON].name = "lamp-on";
-  s->opt[OPT_LAMP_ON].title = SANE_I18N ("Lamp always on");
-  s->opt[OPT_LAMP_ON].desc =
-    SANE_I18N ("Don't turn off the lamp after leaving the frontend.");
-  s->opt[OPT_LAMP_ON].type = SANE_TYPE_BOOL;
-  s->opt[OPT_LAMP_ON].unit = SANE_UNIT_NONE;
-  s->opt[OPT_LAMP_ON].constraint_type = SANE_CONSTRAINT_NONE;
-  s->val[OPT_LAMP_ON].w = SANE_FALSE;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].name = SANE_NAME_LAMP_OFF_AT_EXIT;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].title = SANE_TITLE_LAMP_OFF_AT_EXIT;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].desc = SANE_DESC_LAMP_OFF_AT_EXIT;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].type = SANE_TYPE_BOOL;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].unit = SANE_UNIT_NONE;
+  s->opt[OPT_LAMP_OFF_AT_EXIT].constraint_type = SANE_CONSTRAINT_NONE;
+  s->val[OPT_LAMP_OFF_AT_EXIT].w = SANE_TRUE;
   if (s->dev->model->is_cis && !(s->dev->model->flags & GT68XX_FLAG_CIS_LAMP))
-    DISABLE (OPT_LAMP_ON);
+    DISABLE (OPT_LAMP_OFF_AT_EXIT);
 
   /* bit depth */
   s->opt[OPT_BIT_DEPTH].name = SANE_NAME_BIT_DEPTH;
@@ -503,7 +522,7 @@ init_options (GT68xx_Scanner * s)
     return SANE_STATUS_NO_MEM;
   dpi_list[0] = count;
   for (count = 0; model->ydpi_values[count] != 0; count++)
-    dpi_list[count + 1] = model->ydpi_values[count];
+    dpi_list[dpi_list[0] - count] = model->ydpi_values[count];
   s->opt[OPT_RESOLUTION].name = SANE_NAME_SCAN_RESOLUTION;
   s->opt[OPT_RESOLUTION].title = SANE_TITLE_SCAN_RESOLUTION;
   s->opt[OPT_RESOLUTION].desc = SANE_DESC_SCAN_RESOLUTION;
@@ -513,6 +532,13 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_RESOLUTION].constraint.word_list = dpi_list;
   s->val[OPT_RESOLUTION].w = 300;
 
+  /* backtrack */
+  s->opt[OPT_BACKTRACK].name = SANE_NAME_BACKTRACK;
+  s->opt[OPT_BACKTRACK].title = SANE_TITLE_BACKTRACK;
+  s->opt[OPT_BACKTRACK].desc = SANE_DESC_BACKTRACK;
+  s->opt[OPT_BACKTRACK].type = SANE_TYPE_BOOL;
+  s->val[OPT_BACKTRACK].w = SANE_FALSE;
+
   /* "Debug" group: */
   s->opt[OPT_DEBUG_GROUP].title = SANE_I18N ("Debugging Options");
   s->opt[OPT_DEBUG_GROUP].desc = "";
@@ -528,7 +554,7 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_AUTO_WARMUP].title = SANE_I18N ("Automatic warmup");
   s->opt[OPT_AUTO_WARMUP].desc = 
     SANE_I18N ("Warm-up until the lamp's brightness is constant "
-	       "instead of insisting on 30 seconds warm-up time.");
+	       "instead of insisting on 60 seconds warm-up time.");
   s->opt[OPT_AUTO_WARMUP].type = SANE_TYPE_BOOL;
   s->opt[OPT_AUTO_WARMUP].unit = SANE_UNIT_NONE;
   s->opt[OPT_AUTO_WARMUP].constraint_type = SANE_CONSTRAINT_NONE;
@@ -541,7 +567,7 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_FULL_SCAN].title = SANE_I18N ("Full scan");
   s->opt[OPT_FULL_SCAN].desc = 
     SANE_I18N ("Scan the complete scanning area including calibration strip. "
-	       "Be carefull. Don't select the full height. For testing only.");
+	       "Be careful. Don't select the full height. For testing only.");
   s->opt[OPT_FULL_SCAN].type = SANE_TYPE_BOOL;
   s->opt[OPT_FULL_SCAN].unit = SANE_UNIT_NONE;
   s->opt[OPT_FULL_SCAN].constraint_type = SANE_CONSTRAINT_NONE;
@@ -595,7 +621,7 @@ init_options (GT68xx_Scanner * s)
   s->opt[OPT_FAST_PREVIEW].name = "fast-preview";
   s->opt[OPT_FAST_PREVIEW].title = SANE_I18N ("Fast preview");
   s->opt[OPT_FAST_PREVIEW].desc =
-    SANE_I18N ("Request that all previews are done in in the fastest "
+    SANE_I18N ("Request that all previews are done in the fastest "
 	       "(low-quality) mode. This may be a non-color mode or a low "
 	       "resolution mode.");
   s->opt[OPT_FAST_PREVIEW].type = SANE_TYPE_BOOL;
@@ -603,6 +629,25 @@ init_options (GT68xx_Scanner * s)
   if (!debug_options)
     DISABLE (OPT_FAST_PREVIEW);
 
+  /* backtrack lines */
+  s->opt[OPT_BACKTRACK_LINES].name = "backtrack-lines";
+  s->opt[OPT_BACKTRACK_LINES].title = SANE_I18N("Backtrack lines");
+  s->opt[OPT_BACKTRACK_LINES].desc = 
+    SANE_I18N ("Number of lines the scan slider moves back when backtracking "
+	       "occurs. That happens when the scanner scans faster than the "
+	       "computer can receive the data. Low values cause faster scans "
+	       "but increase the risk of omitting lines.");
+  s->opt[OPT_BACKTRACK_LINES].type = SANE_TYPE_INT;
+  s->opt[OPT_BACKTRACK_LINES].unit = SANE_UNIT_NONE;
+  s->opt[OPT_BACKTRACK_LINES].constraint_type = SANE_CONSTRAINT_RANGE;
+  s->opt[OPT_BACKTRACK_LINES].constraint.range = &u8_range;
+  if (s->dev->model->is_cis)
+    s->val[OPT_BACKTRACK_LINES].w = 0x10;
+  else
+    s->val[OPT_BACKTRACK_LINES].w = 0x3f;
+  if (!debug_options)
+    DISABLE (OPT_BACKTRACK_LINES);
+
   /* "Enhancement" group: */
   s->opt[OPT_ENHANCEMENT_GROUP].title = SANE_I18N ("Enhancement");
   s->opt[OPT_ENHANCEMENT_GROUP].desc = "";
@@ -633,32 +678,6 @@ init_options (GT68xx_Scanner * s)
   s->val[OPT_THRESHOLD].w = 128;
   DISABLE (OPT_THRESHOLD);
 
-  /* gain correction */
-  s->opt[OPT_GAIN].name = "gain";
-  s->opt[OPT_GAIN].title = 
-    SANE_I18N ("Gain correction");
-  s->opt[OPT_GAIN].desc =
-    SANE_I18N ("This value is added to the internal gain value. "
-	       "Use for extremely light or dark images.");
-  s->opt[OPT_GAIN].type = SANE_TYPE_INT;
-  s->opt[OPT_GAIN].unit = SANE_UNIT_NONE;
-  s->opt[OPT_GAIN].constraint_type = SANE_CONSTRAINT_RANGE;
-  s->opt[OPT_GAIN].constraint.range = &offset_range;
-  s->val[OPT_GAIN].w = 0;
-
-  /* offset correction */
-  s->opt[OPT_OFFSET].name = "offset";
-  s->opt[OPT_OFFSET].title = 
-    SANE_I18N ("Offset correction");
-  s->opt[OPT_OFFSET].desc =
-    SANE_I18N ("This value is added to the internal offset value. "
-	       "Use for extremely light or dark images.");
-  s->opt[OPT_OFFSET].type = SANE_TYPE_INT;
-  s->opt[OPT_OFFSET].unit = SANE_UNIT_NONE;
-  s->opt[OPT_OFFSET].constraint_type = SANE_CONSTRAINT_RANGE;
-  s->opt[OPT_OFFSET].constraint.range = &offset_range;
-  s->val[OPT_OFFSET].w = 0;
-
   /* "Geometry" group: */
   s->opt[OPT_GEOMETRY_GROUP].title = SANE_I18N ("Geometry");
   s->opt[OPT_GEOMETRY_GROUP].desc = "";
@@ -759,12 +778,20 @@ attach (SANE_String_Const devname, GT68xx_Device ** devp, SANE_Bool may_wait)
 
   if (!gt68xx_device_is_configured (dev))
     {
-      DBG (4, "attach: device `%s' is not supported: %s\n", devname,
-	   sane_strstatus (status));
-      gt68xx_device_free (dev);
-      if (devp)
-	*devp = 0;
-      return status;
+      GT68xx_Model * model;
+      DBG (2, "attach: Warning: device `%s' is not listed in device table\n", devname);
+      DBG (2, "attach: If you have manually added it, use override in gt68xx.conf\n");
+      gt68xx_device_get_model ("unknown-scanner", &model);
+      status = gt68xx_device_set_model (dev, model);
+      if (status != SANE_STATUS_GOOD)
+	{
+	  DBG (4, "attach: couldn't set model: %s\n", sane_strstatus (status));
+	  gt68xx_device_free (dev);
+	  if (devp)
+	    *devp = 0;
+	  return status;
+	}
+      dev->manual_selection = SANE_TRUE;
     }
 
   dev->file_name = strdup (devname);
@@ -1267,6 +1294,33 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
       DBG (0, "         scanner and what does (not) work.\n");
     }
 
+  if (dev->manual_selection)
+    {
+      DBG (0, "WARNING: You have manually added the ids of your scanner \n");
+      DBG (0, "         to gt68xx.conf. Please use an appropriate override \n");
+      DBG (0, "         for your scanner. Use extreme care and switch off \n");
+      DBG (0, "         the scanner immediately if you hear unusual noise. \n");
+      DBG (0, "         Please report any success to \n");
+      DBG (0, "         henning@meier-geinitz.de. Please provide as many\n");
+      DBG (0, "         details as possible, e.g. the exact name of your\n");
+      DBG (0, "         scanner, ids, settings etc.\n");
+      
+      if (strcmp (dev->model->name, "unknown-scanner") == 0)
+	{
+	  GT68xx_USB_Device_Entry *entry;
+
+	  DBG (0, "ERROR: You haven't chosen an override in gt68xx.conf. Please use \n");
+	  DBG (0, "       one of the following: \n");
+
+	  for (entry = gt68xx_usb_device_list; entry->model; ++entry)
+	    {
+	      if (strcmp (entry->model->name, "unknown-scanner") != 0)
+		DBG (0, "       %s\n", entry->model->name);
+	    }
+	  return SANE_STATUS_UNSUPPORTED;
+	}
+    }
+
   /* The firmware check is disabled by default because it may confuse
      some scanners: So the firmware is loaded everytime. */
 #if 0
@@ -1314,6 +1368,7 @@ sane_open (SANE_String_Const devicename, SANE_Handle * handle)
   s->first_scan = SANE_TRUE;
   s->gamma_table = 0;
   RIE (init_options (s));
+  dev->gray_mode_color = 0x02;
 
   DBG (5, "sane_open: exit\n");
 
@@ -1324,6 +1379,7 @@ void
 sane_close (SANE_Handle handle)
 {
   GT68xx_Scanner *prev, *s;
+  GT68xx_Device *dev;
 
   DBG (5, "sane_close: start\n");
 
@@ -1346,11 +1402,13 @@ sane_close (SANE_Handle handle)
   else
     first_handle = s->next;
 
-  if (s->val[OPT_LAMP_ON].w == SANE_FALSE)
+  if (s->val[OPT_LAMP_OFF_AT_EXIT].w == SANE_TRUE)
     gt68xx_device_lamp_control (s->dev, SANE_FALSE, SANE_FALSE);
-  gt68xx_device_deactivate (s->dev);
-  gt68xx_device_close (s->dev);
+
+  dev = s->dev;
   gt68xx_scanner_free (s);
+  gt68xx_device_deactivate (dev);
+  gt68xx_device_close (dev);
 
   DBG (5, "sane_close: exit\n");
 }
@@ -1418,12 +1476,12 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	case OPT_FULL_SCAN:
 	case OPT_COARSE_CAL:
 	case OPT_COARSE_CAL_ONCE:
-	case OPT_OFFSET:
-	case OPT_GAIN:
 	case OPT_QUALITY_CAL:
 	case OPT_FAST_PREVIEW:
+	case OPT_BACKTRACK:
+	case OPT_BACKTRACK_LINES:
 	case OPT_PREVIEW:
-	case OPT_LAMP_ON:
+	case OPT_LAMP_OFF_AT_EXIT:
 	case OPT_AUTO_WARMUP:
 	case OPT_GAMMA_VALUE:
 	case OPT_THRESHOLD:
@@ -1435,8 +1493,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  break;
 	  /* string options: */
 	case OPT_MODE:
-	  strcpy (val, s->val[option].s);
-	  break;
+	case OPT_GRAY_MODE_COLOR:
 	case OPT_SOURCE:
 	  strcpy (val, s->val[option].s);
 	  break;
@@ -1477,16 +1534,23 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  RIE (calc_parameters (s));
 	  myinfo |= SANE_INFO_RELOAD_PARAMS;
 	  break;
-	case OPT_LAMP_ON:
+	case OPT_LAMP_OFF_AT_EXIT:
 	case OPT_AUTO_WARMUP:
 	case OPT_COARSE_CAL_ONCE:
-	case OPT_OFFSET:
-	case OPT_GAIN:
+	case OPT_BACKTRACK_LINES:
 	case OPT_QUALITY_CAL:
 	case OPT_GAMMA_VALUE:
 	case OPT_THRESHOLD:
 	  s->val[option].w = *(SANE_Word *) val;
 	  break;
+	case OPT_GRAY_MODE_COLOR:
+	  if (strcmp (s->val[option].s, val) != 0)
+	    {			/* something changed */
+	      if (s->val[option].s)
+		free (s->val[option].s);
+	      s->val[option].s = strdup (val);
+	    }
+	  break;
 	case OPT_SOURCE:
 	  if (strcmp (s->val[option].s, val) != 0)
 	    {			/* something changed */
@@ -1515,18 +1579,25 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  if (s->val[option].s)
 	    free (s->val[option].s);
 	  s->val[option].s = strdup (val);
-	  if (strcmp (s->val[option].s, "Lineart") == 0)
+	  if (strcmp (s->val[option].s, SANE_VALUE_SCAN_MODE_LINEART) == 0)
 	    {
 	      ENABLE (OPT_THRESHOLD);
 	      DISABLE (OPT_BIT_DEPTH);
+	      ENABLE (OPT_GRAY_MODE_COLOR);
 	    }
 	  else
 	    {
 	      DISABLE (OPT_THRESHOLD);
-	      if (strcmp (s->val[option].s, "Gray") == 0)
-		RIE (create_bpp_list (s, s->dev->model->bpp_gray_values));
+	      if (strcmp (s->val[option].s, SANE_VALUE_SCAN_MODE_GRAY) == 0)
+		{
+		  RIE (create_bpp_list (s, s->dev->model->bpp_gray_values));
+		  ENABLE (OPT_GRAY_MODE_COLOR);
+		}
 	      else
-		RIE (create_bpp_list (s, s->dev->model->bpp_color_values));
+		{
+		  RIE (create_bpp_list (s, s->dev->model->bpp_color_values));
+		  DISABLE (OPT_GRAY_MODE_COLOR);
+		}
 	      if (s->bpp_list[0] < 2)
 		DISABLE (OPT_BIT_DEPTH);
 	      else
@@ -1550,6 +1621,15 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
 	  myinfo |= SANE_INFO_RELOAD_OPTIONS;
 	  break;
 
+	case OPT_BACKTRACK:
+	  s->val[option].w = *(SANE_Word *) val;
+	  if (s->val[option].w == SANE_TRUE)
+	    ENABLE (OPT_BACKTRACK_LINES);
+	  else
+	    DISABLE (OPT_BACKTRACK_LINES);
+	  myinfo |= SANE_INFO_RELOAD_OPTIONS;
+	  break;
+
 	default:
 	  DBG (2, "sane_control_option: can't set unknown option %d\n",
 	       option);
@@ -1580,6 +1660,13 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters * params)
   if (params)
     *params = s->params;
 
+  DBG (4, "sane_get_parameters: format=%d, last_frame=%d, lines=%d\n",
+       s->params.format, s->params.last_frame, s->params.lines);
+  DBG (4, "sane_get_parameters: pixels_per_line=%d, bytes per line=%d\n",
+       s->params.pixels_per_line, s->params.bytes_per_line);
+  DBG (3, "sane_get_parameters: pixels %dx%dx%d\n",
+       s->params.pixels_per_line, s->params.lines, 1 << s->params.depth);
+
   DBG (5, "sane_get_parameters: exit\n");
 
   return SANE_STATUS_GOOD;
@@ -1612,6 +1699,13 @@ sane_start (SANE_Handle handle)
       return SANE_STATUS_INVAL;
     }
 
+  if (strcmp (s->val[OPT_GRAY_MODE_COLOR].s, GT68XX_COLOR_BLUE) == 0)
+    s->dev->gray_mode_color = 0x01;
+  else if (strcmp (s->val[OPT_GRAY_MODE_COLOR].s, GT68XX_COLOR_GREEN) == 0)
+    s->dev->gray_mode_color = 0x02;
+  else
+    s->dev->gray_mode_color = 0x03;
+    
   setup_scan_request (s, &scan_request);
   if (!s->first_scan && s->val[OPT_COARSE_CAL_ONCE].w == SANE_TRUE)
     s->auto_afe = SANE_FALSE;
@@ -1650,6 +1744,21 @@ sane_start (SANE_Handle handle)
   gt68xx_scanner_wait_for_positioning (s);
   gettimeofday (&s->start_time, 0);
 
+  if (s->val[OPT_BACKTRACK].w == SANE_TRUE)
+    scan_request.backtrack = SANE_TRUE;
+  else
+    {
+      if (s->val[OPT_RESOLUTION].w >= s->dev->model->ydpi_no_backtrack)
+	scan_request.backtrack = SANE_FALSE;
+      else
+	scan_request.backtrack = SANE_TRUE;
+    }
+    
+  if (scan_request.backtrack)
+    scan_request.backtrack_lines = s->val[OPT_BACKTRACK_LINES].w;
+  else
+    scan_request.backtrack_lines = 0;
+
   RIE (gt68xx_scanner_calibrate (s, &scan_request));
   RIE (gt68xx_scanner_start_scan (s, &scan_request, &scan_params));
   for (i = 0; i < scan_params.overscan_lines; ++i)
@@ -1664,7 +1773,15 @@ sane_start (SANE_Handle handle)
   s->byte_count = s->reader->params.pixel_xs;
   s->total_bytes = 0;
   s->first_scan = SANE_FALSE;
+
+#ifdef DEBUG_BRIGHTNESS
+  s->average_white = 0;
+  s->max_white = 0;
+  s->min_black = 255;
+#endif
+
   s->scanning = SANE_TRUE;
+
   DBG (5, "sane_start: exit\n");
   return SANE_STATUS_GOOD;
 }
@@ -1724,7 +1841,7 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
   else
     inflate_x = 1;
 
-  lineart = (strcmp (s->val[OPT_MODE].s, "Lineart") == 0)
+  lineart = (strcmp (s->val[OPT_MODE].s, SANE_VALUE_SCAN_MODE_LINEART) == 0)
     ? SANE_TRUE : SANE_FALSE;
 
   if (s->reader->params.color)
@@ -1826,6 +1943,11 @@ sane_read (SANE_Handle handle, SANE_Byte * buf, SANE_Int max_len,
 	      buf[*len] = (buffer_pointers[color][s->byte_count] >> 8) & 0xff;
 	      if (s->total_bytes % (inflate_x * 3) == (inflate_x * 3 - 1))
 		s->byte_count++;
+#ifdef DEBUG_BRIGHTNESS
+	      s->average_white += buf[*len];
+	      s->max_white = (buf[*len] > s->max_white) ? buf[*len] : s->max_white;
+	      s->min_black = (buf[*len] < s->min_black) ? buf[*len] : s->min_black;
+#endif
 	    }
 	}
       else
@@ -1895,6 +2017,11 @@ sane_cancel (SANE_Handle handle)
 	  DBG (3,
 	       "sane_cancel: scan finished, scanned %d bytes in %d seconds\n",
 	       s->total_bytes, secs);
+#ifdef DEBUG_BRIGHTNESS
+	  DBG (1, "sane_cancel: average white: %d, max_white=%d, min_black=%d\n",
+	       s->average_white / s->total_bytes, s->max_white, s->min_black);
+#endif
+
 	}
       gt68xx_scanner_stop_scan (s);
       gt68xx_scanner_wait_for_positioning (s);
