@@ -62,7 +62,6 @@
 #include "../include/sane/sane.h"
 #include "../include/sane/saneopts.h"
 #include "../include/sane/sanei_scsi.h"
-#include "ibm.h"
 
 #define BACKEND_NAME ibm
 #include "../include/sane/sanei_backend.h"
@@ -74,6 +73,9 @@
 #include "../include/sane/sanei_config.h"
 #define IBM_CONFIG_FILE "ibm.conf"
 
+#include "ibm.h"
+#include "ibm-scsi.c"
+
 #define MM_PER_INCH	25.4
 #define MAX(a,b)	((a) > (b) ? (a) : (b))
 
@@ -82,7 +84,6 @@ static Ibm_Device *first_dev = NULL;
 static Ibm_Scanner *first_handle = NULL;
 /* static int is50 = 0; */
 
-#include "ibm-scsi.c"
 
 static size_t
 max_string_size (const SANE_String_Const strings[])
@@ -538,7 +539,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   FILE *fp;
 
   DBG_INIT ();
-  DBG (11, ">> sane_init (authorize = %p)\n", authorize);
+  DBG (11, ">> sane_init (authorize = %p)\n", (void *) authorize);
 
 #if defined PACKAGE && defined VERSION
   DBG (2, "sane_init: ibm backend version %d.%d-%d ("
@@ -1175,7 +1176,7 @@ SANE_Status
 sane_get_select_fd (SANE_Handle handle, SANE_Int * fd)
 {
   DBG (5, ">> sane_get_select_fd (handle = %p, fd = %p)\n",
-       handle, fd);
+       handle, (void *) fd);
   DBG (5, "<< sane_get_select_fd\n");
 
   return SANE_STATUS_UNSUPPORTED;
