@@ -43,7 +43,7 @@
 
 Userspace scan tool for the Microtek 3600 scanner
 
-$Id: sm3600-scanmtek.c,v 1.9 2003/04/02 14:04:06 hmg Exp $
+$Id: sm3600-scanmtek.c,v 1.11 2004/04/15 21:06:58 eichholz-guest Exp $
 
 ====================================================================== */
 
@@ -272,3 +272,39 @@ TState UploadGammaTable(TInstance *this, int iByteAddress, SANE_Int *pnGamma)
   free(puchGamma);
   return rc;
 }
+
+/* **********************************************************************
+
+UploadGainCorrection()
+
+********************************************************************** */
+
+__SM3600EXPORT__
+TState UploadGainCorrection(TInstance *this, int iTableOffset)
+{
+  {
+    struct TGain {
+      unsigned char uchLow;
+      unsigned char uchHigh;
+    } aGain[0x2000]; /* 16 KB */
+    int i,iOff;
+    unsigned short uwGain;
+
+    /*
+      Oopsi: correction data starts at the left of the scanning window!
+    */
+    iOff=this->param.x/2+this->calibration.xMargin;
+    memset(aGain,0xFF,sizeof(aGain));
+    RegWrite(this,0x3D,1,0x0F | 0x80); /* 10XXXXXX : one offset table */
+    RegWrite(this,0x3F,1, iTableOffset==0x6000 ? 0x18 : 0x08); /* 16KB gain at 0x06000 or 0x02000 */
+    for (i=iOff; i<MAX_PIXEL_PER_SCANLINE; i++)
+      {
+	uwGain=this->calibration.achStripeY[i]<<4;
+	aGain[i-iOff].uchLow =(unsigned char)(uwGain&0xFF);
+	aGain[i-iOff].uchHigh=(unsigned char)(uwGain>>8);
+      }
+    for (i=0; i<0x4000; i+=0x1000)
+      MemWriteArray(this,(iTableOffset+i)>>1,0x1000,((unsigned char*)aGain)+i);
+  }
+  return SANE_STATUS_GOOD;
+}
