@@ -167,7 +167,7 @@ InitOptions(TInstance *this)
       static double afFullBed[] = { 22.0,30.0, 50.0, 80.0 }; /* TODO: calculate exactly! */
       static const SANE_Range *aRangesXY[] = { &rangeXmm,&rangeYmm,&rangeXmm,&rangeYmm };
       SANE_Option_Descriptor *pdesc;
-      TOptionValue           *pval;
+      Option_Value           *pval;
       /* shorthands */
       pdesc=this->aoptDesc+iOpt;
       pval=this->aoptVal+iOpt;
@@ -683,7 +683,7 @@ sane_get_parameters (SANE_Handle handle, SANE_Parameters *p)
   GetAreaSize(this);
   p->pixels_per_line=this->state.cxPixel;
   /* TODO: we need a more stable cyPixel prediction */
-  p->lines=this->state.cyPixel+1;
+  p->lines=this->state.cyPixel;
   p->last_frame=SANE_TRUE;
   switch (this->mode)
     {
