@@ -46,7 +46,7 @@
    This file is a component of the implementation of a backend for many
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
-/* $Id: snapscan-options.c,v 1.16 2004/04/08 21:53:10 oliver-guest Exp $
+/* $Id: snapscan-options.c,v 1.17 2004/09/02 20:59:11 oliver-guest Exp $
    SANE SnapScan backend */
 
 /* default option values */
@@ -154,6 +154,16 @@ static const SANE_Range y_range_tpo_1670 =
     SANE_FIX (0.0), SANE_FIX (228.0), 0
 };        /* mm */
 
+/* TPO range for the Epson 2480 */
+static const SANE_Range x_range_tpo_2480 =
+{
+    SANE_FIX (0.0), SANE_FIX (55.0), 0
+};        /* mm */
+static const SANE_Range y_range_tpo_2480 =
+{
+    SANE_FIX (0.0), SANE_FIX (125.0), 0
+};        /* mm */
+
 static SANE_Range x_range_tpo;
 static SANE_Range y_range_tpo;
 static const SANE_Range gamma_range =
@@ -209,6 +219,8 @@ static void init_options (SnapScan_Scanner * ps)
         {10, 50, 75, 100, 150, 200, 300, 450, 600, 900, 1200};
     static SANE_Word resolutions_1600[] =
         {10, 50, 75, 100, 150, 200, 300, 400, 600, 800, 1600};
+    static SANE_Word resolutions_2400[] =
+        {10, 50, 75, 100, 150, 200, 300, 400, 600, 1200, 2400};
     static SANE_String_Const names_all[] =
         {md_colour, md_bilevelcolour, md_greyscale, md_lineart, NULL};
     static SANE_String_Const names_basic[] =
@@ -236,6 +248,10 @@ static void init_options (SnapScan_Scanner * ps)
         x_range_tpo = x_range_tpo_1670;
         y_range_tpo = y_range_tpo_1670;
         break;
+    case PERFECTION2480:
+        x_range_tpo = x_range_tpo_2480;
+        y_range_tpo = y_range_tpo_2480;
+        break;
     default:
         x_range_tpo = x_range_tpo_default;
         y_range_tpo = y_range_tpo_default;
@@ -289,6 +305,9 @@ static void init_options (SnapScan_Scanner * ps)
     case PERFECTION1670:
         po[OPT_SCANRES].constraint.word_list = resolutions_1600;
         break;
+    case PERFECTION2480:
+        po[OPT_SCANRES].constraint.word_list = resolutions_2400;
+        break;
     default:
         po[OPT_SCANRES].constraint.word_list = resolutions_600;
         break;
@@ -506,6 +525,7 @@ static void init_options (SnapScan_Scanner * ps)
     if ((!(ps->hconfig & HCFG_CAL_ALLOWED))
         || (ps->pdev->model == SNAPSCANE52)
         || (ps->pdev->model == PERFECTION1670)
+        || (ps->pdev->model == PERFECTION2480)
         || (ps->pdev->model == PRISA5300)) {
         po[OPT_QUALITY_CAL].cap |= SANE_CAP_INACTIVE;
         ps->val[OPT_QUALITY_CAL].b = SANE_FALSE;
@@ -1488,6 +1508,9 @@ SANE_Status sane_control_option (SANE_Handle h,
 
 /*
  * $Log: snapscan-options.c,v $
+ * Revision 1.17  2004/09/02 20:59:11  oliver-guest
+ * Added support for Epson 2480
+ *
  * Revision 1.16  2004/04/08 21:53:10  oliver-guest
  * Use sanei_thread in snapscan backend
  *
