@@ -68,7 +68,7 @@
      So far this strategy has worked flawlessly. Thanks Dmitri!
 */
 
-/* $Id: snapscan-usb.c,v 1.15 2004/04/09 11:59:02 oliver-guest Exp $
+/* $Id: snapscan-usb.c,v 1.19 2004/10/03 17:34:36 hmg-guest Exp $
    SnapScan backend scan data sources */
 
 #include "snapscan-usb.h"
@@ -79,8 +79,13 @@
 static snapscan_mutex_t sem_id;
 static sense_handler_type usb_sense_handler;
 static void* usb_pss;
-static unsigned long read_urbs = 0;
-static unsigned long write_urbs = 0;
+
+struct urb_counters_t {
+    unsigned long read_urbs;
+    unsigned long write_urbs;
+};
+
+static struct urb_counters_t* urb_counters = NULL;
 
 /* Forward declarations */
 static SANE_Status usb_request_sense(SnapScan_Scanner *pss);
@@ -91,8 +96,8 @@ static SANE_Status snapscani_usb_cmd(int fd, const void *src, size_t src_size,
     static const char me[] = "snapscani_usb_cmd";
     int status;
 
-    DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-         fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
+    DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+         fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
     while(bqhead) {
         status = atomic_usb_cmd(fd, bqhead->src, bqhead->src_size, NULL, NULL);
@@ -126,8 +131,8 @@ static SANE_Status atomic_usb_cmd(int fd, const void *src, size_t src_size,
     int status;
     sigset_t all,oldset;
 
-    DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-         fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
+    DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+         fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
     /* Prevent the calling process from being killed */
     sigfillset(&all);
@@ -160,46 +165,64 @@ static SANE_Status snapscani_usb_open(const char *dev, int *fdp,
     }
     usb_sense_handler=sense_handler;
     usb_pss = pss;
-    read_urbs = 0;
-    write_urbs = 0;    
+    urb_counters->read_urbs = 0;
+    urb_counters->write_urbs = 0;    
     return sanei_usb_open(dev, fdp);
 }
 
 
 static void snapscani_usb_close(int fd) {
     static const char me[] = "snapscani_usb_close";
-
+    SANE_Word vendor_id, product_id;
+    
     DBG (DL_CALL_TRACE, "%s(%d)\n", me, fd);
-    DBG (DL_DATA_TRACE,"1st read %ld write %ld\n", read_urbs, write_urbs);
-    if ((read_urbs & 0x01) && (write_urbs & 0x01))
+    DBG (DL_DATA_TRACE,"1st read %ld write %ld\n", urb_counters->read_urbs, urb_counters->write_urbs);
+    
+    /* Check if URB counting is needed. If yes, ensure the number of sent and
+       received URBs is even.
+       Odd number of URBs only cause problems with libusb and certain 
+       scanner models. On other scanner models, sending additional commands
+       seems to cause problems (e.g. 1212u_2).
+       If sanei_usb_get_vendor_product returns an error there's probably no
+       libusb, so everything's fine.
+    */
+    if (sanei_usb_get_vendor_product(fd, &vendor_id, &product_id) == SANE_STATUS_GOOD)
     {
-        char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
-
-        usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
+        /* Exclude 1212u_2 */
+        if (!((vendor_id == USB_VENDOR_AGFA) && (product_id == USB_PRODUCT_1212U2)))        
+        {
+            if ((urb_counters->read_urbs & 0x01) && (urb_counters->write_urbs & 0x01))
+            {
+                char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
+        
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
+            }
+            else if (urb_counters->read_urbs & 0x01)
+            {
+                size_t read_bytes;
+                char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
+                char cmd2[] = {INQUIRY, 0, 0, 0, 120, 0};
+                char data[120];
+        
+                read_bytes = 120;
+                snapscani_usb_cmd (fd, cmd2, sizeof (cmd2), data, &read_bytes);
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
+            }
+            else if (urb_counters->write_urbs & 0x01)
+            {
+                size_t read_bytes;
+                char cmd[] = {INQUIRY, 0, 0, 0, 120, 0};
+                char data[120];
+        
+                read_bytes = 120;
+                snapscani_usb_cmd (fd, cmd, sizeof (cmd), data, &read_bytes);
+            }
+            DBG (DL_DATA_TRACE,"2nd read %ld write %ld\n", urb_counters->read_urbs, 
+                urb_counters->write_urbs);
+        }
     }
-    else if (read_urbs & 0x01)
-    {
-        size_t read_bytes;
-        char cmd[] = {TEST_UNIT_READY, 0, 0, 0, 0, 0};
-        char cmd2[] = {INQUIRY, 0, 0, 0, 120, 0};
-        char data[120];
-
-        read_bytes = 120;
-        snapscani_usb_cmd (fd, cmd2, sizeof (cmd2), data, &read_bytes);
-        usb_cmd (fd, cmd, sizeof (cmd), NULL, 0);
-    }
-    else if (write_urbs & 0x01)
-    {
-        size_t read_bytes;
-        char cmd[] = {INQUIRY, 0, 0, 0, 120, 0};
-        char data[120];
-
-        read_bytes = 120;
-        usb_cmd (fd, cmd, sizeof (cmd), data, &read_bytes);
-    }
-    DBG (DL_DATA_TRACE,"2nd read %ld write %ld\n", read_urbs, write_urbs);
-    read_urbs = 0;
-    write_urbs = 0;
+    urb_counters->read_urbs = 0;
+    urb_counters->write_urbs = 0;
     snapscani_mutex_close(&sem_id);
     sanei_usb_close(fd);
 }
@@ -252,11 +275,11 @@ static SANE_Status usb_write(int fd, const void *buf, size_t n) {
 
     status = sanei_usb_write_bulk(fd, (const SANE_Byte*)buf, &bytes_written);
     if(bytes_written != n) {
-        DBG (DL_MAJOR_ERROR, "%s Only %d bytes written\n",me,bytes_written);
+      DBG (DL_MAJOR_ERROR, "%s Only %lu bytes written\n",me, (u_long) bytes_written);
         status = SANE_STATUS_IO_ERROR;
     }
-    write_urbs += (bytes_written + 7) / 8;
-    DBG (DL_DATA_TRACE, "Written %d bytes\n", bytes_written);
+    urb_counters->write_urbs += (bytes_written + 7) / 8;
+    DBG (DL_DATA_TRACE, "Written %lu bytes\n", (u_long) bytes_written);
     return status;
 }
 
@@ -268,12 +291,12 @@ static SANE_Status usb_read(SANE_Int fd, void *buf, size_t n) {
 
     status = sanei_usb_read_bulk(fd, (SANE_Byte*)buf, &bytes_read);
     if (bytes_read != n) {
-        DBG (DL_MAJOR_ERROR, "%s Only %d bytes read\n",me,bytes_read);
+        DBG (DL_MAJOR_ERROR, "%s Only %lu bytes read\n",me, (u_long) bytes_read);
         status = SANE_STATUS_IO_ERROR;
     }
-    read_urbs += ((63 + bytes_read) / 64); 
+    urb_counters->read_urbs += ((63 + bytes_read) / 64); 
     DBG(DL_DATA_TRACE, "%s: reading: %s\n",me,usb_debug_data(dbgmsg,buf,n));
-    DBG(DL_DATA_TRACE, "Read %d bytes\n", bytes_read);
+    DBG(DL_DATA_TRACE, "Read %lu bytes\n", (u_long) bytes_read);
     return status;
 }
 
@@ -321,9 +344,8 @@ static SANE_Status usb_cmd(int fd, const void *src, size_t src_size,
   int status,tstatus;
   int cmdlen,datalen;
 
-  DBG (DL_CALL_TRACE, "%s(%d,0x%x,%d,0x%x,0x%x (%d))\n", me,
-       fd,(int)src,src_size,(int)dst,(int)dst_size,dst_size ? *dst_size : 0);
-
+  DBG (DL_CALL_TRACE, "%s(%d,0x%lx,%lu,0x%lx,0x%lx (%lu))\n", me,
+       fd, (u_long) src,(u_long) src_size,(u_long) dst, (u_long) dst_size,(u_long) (dst_size ? *dst_size : 0));
 
   /* Since the  "Send Diagnostic" command isn't supported by
      all Snapscan USB-scanners it's disabled .
@@ -394,7 +416,7 @@ static int enqueue_bq(int fd,const void *src, size_t src_size)
     static const char me[] = "enqueue_bq";
     struct usb_busy_queue *bqe;
 
-    DBG (DL_CALL_TRACE, "%s(%d,%p,%d)\n", me, fd,src,src_size);
+    DBG (DL_CALL_TRACE, "%s(%d,%p,%lu)\n", me, fd,src, (u_long) src_size);
 
     if((bqe = malloc(sizeof(struct usb_busy_queue))) == NULL)
         return -1;
@@ -471,8 +493,88 @@ static SANE_Status usb_request_sense(SnapScan_Scanner *pss) {
     return status;
 }
 
+#if defined USE_PTHREAD || defined HAVE_OS2_H
+static SANE_Status snapscani_usb_shm_init(void)
+{
+    unsigned int shm_size = sizeof(struct urb_counters_t);
+    urb_counters = (struct urb_counters_t*) malloc(shm_size);
+    if (urb_counters == NULL)
+    {
+        return SANE_STATUS_NO_MEM;
+    }
+    memset(urb_counters, 0, shm_size);
+    return SANE_STATUS_GOOD;
+        
+}
+
+static void snapscani_usb_shm_exit(void)
+{
+    if (urb_counters)
+    {
+        free ((void*)urb_counters);
+        urb_counters = NULL;
+    }
+}
+#else
+#include <sys/ipc.h>
+#include <sys/shm.h>
+static SANE_Status snapscani_usb_shm_init(void)
+{
+    unsigned int shm_size = sizeof(struct urb_counters_t);
+    void* shm_area = NULL;
+    int shm_id = shmget (IPC_PRIVATE, shm_size, IPC_CREAT | SHM_R | SHM_W);
+    if (shm_id == -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot create shared memory segment: %s\n",
+            strerror (errno));
+        return SANE_STATUS_NO_MEM;
+    }
+    
+    shm_area = shmat (shm_id, NULL, 0);
+    if (shm_area == (void *) -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot attach to shared memory segment: %s\n",
+            strerror (errno));
+        shmctl (shm_id, IPC_RMID, NULL);
+        return SANE_STATUS_NO_MEM;
+    }
+    
+    if (shmctl (shm_id, IPC_RMID, NULL) == -1)
+    {
+        DBG (DL_MAJOR_ERROR, "snapscani_usb_shm_init: cannot remove shared memory segment id: %s\n",
+            strerror (errno));
+        shmdt (shm_area);
+        shmctl (shm_id, IPC_RMID, NULL);
+        return SANE_STATUS_NO_MEM;
+    }
+    urb_counters = (struct urb_counters_t*) shm_area;
+    memset(urb_counters, 0, shm_size);
+    return SANE_STATUS_GOOD;
+}
+
+static void snapscani_usb_shm_exit(void)
+{
+    if (urb_counters)
+    {
+        shmdt (urb_counters);
+        urb_counters = NULL;
+    }
+}
+#endif
 /*
  * $Log: snapscan-usb.c,v $
+ * Revision 1.19  2004/10/03 17:34:36  hmg-guest
+ * 64 bit platform fixes (bug #300799).
+ *
+ * Revision 1.18  2004/06/16 19:52:26  oliver-guest
+ * Don't enforce even number of URB packages on 1212u_2. Fixes bug #300753.
+ *
+ * Revision 1.17  2004/06/06 14:50:36  oliver-guest
+ * Use shared memory functions only when needed
+ *
+ * Revision 1.16  2004/05/26 22:37:01  oliver-guest
+ * Use shared memory for urb counters in snapscan backend
+ *
  * Revision 1.15  2004/04/09 11:59:02  oliver-guest
  * Fixes for pthread implementation
  *
