@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan.c,v 1.40 2004/04/09 11:59:02 oliver-guest Exp $
+/* $Id: snapscan.c,v 1.45 2004/10/03 17:34:36 hmg-guest Exp $
    SANE SnapScan backend */
 
 #include "../include/sane/config.h"
@@ -79,11 +79,15 @@
 
 #define EXPECTED_MAJOR       1
 #define MINOR_VERSION        4
-#define BUILD               37
+#define BUILD               40
 
 #define BACKEND_NAME snapscan
 
+#ifdef __GNUC__
 #define UNUSEDARG __attribute__ ((unused))
+#else
+#define UNUSEDARG
+#endif
 
 #include "../include/sane/sanei_backend.h"
 #include "../include/sane/saneopts.h"
@@ -175,6 +179,9 @@ static inline int calibration_line_length(SnapScan_Scanner *pss)
     case PERFECTION1670:
         pos_factor = 800;
         break;
+    case PERFECTION2480:
+        pos_factor = 800;
+        break;
     default:
         break;
     }
@@ -556,6 +563,11 @@ static SANE_Status add_usb_device (SANE_String_Const full_name) {
 
     DBG (DL_VERBOSE, "%s: Detected (kind of) an USB device\n", me);
     bus_type = USB;
+    status = snapscani_usb_shm_init();
+    if (status != SANE_STATUS_GOOD)
+    {
+        return status; 
+    }
     status = snapscani_usb_open (name, &fd, sense_handler, NULL);
     if (status != SANE_STATUS_GOOD)
     {
@@ -591,6 +603,8 @@ static SANE_Status add_usb_device (SANE_String_Const full_name) {
         status = snapscani_check_device(fd, bus_type, vendor, model, &model_num);
         snapscani_usb_close(fd);
     }
+    /* deinit shared memory, will be initialized again in open_scanner */
+    snapscani_usb_shm_exit();     
     if (status == SANE_STATUS_GOOD) {
         status = snapscani_init_device_structure(
             &pd,
@@ -674,7 +688,6 @@ SANE_Status sane_init (SANE_Int *version_code,
 
     sanei_usb_init();
     sanei_thread_init();
-
     /* build a device structure */
     fp = sanei_config_open (SNAPSCAN_CONFIG_FILE);
     if (!fp)
@@ -740,7 +753,6 @@ SANE_Status sane_init (SANE_Int *version_code,
         for (i = 0;  i < 64;  i++)
             D8[i] = (u_char) (4 * D8[i] + 2);
     }
-
     return SANE_STATUS_GOOD;
 }
 
@@ -860,8 +872,8 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
                 pss->phys_buf_sz = sanei_scsi_max_request_size;
             }
             DBG (DL_DATA_TRACE,
-                "%s: Allocating %d bytes as scanner buffer.\n",
-                me, pss->phys_buf_sz);
+                "%s: Allocating %lu bytes as scanner buffer.\n",
+                me, (u_long) pss->phys_buf_sz);
             pss->buf = (u_char *) malloc(pss->phys_buf_sz);
             if (!pss->buf) {
                 DBG (DL_MAJOR_ERROR,
@@ -875,7 +887,11 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
                  me,
                  (void *) pss);
         }
-
+        status = snapscani_usb_shm_init();
+        if (status != SANE_STATUS_GOOD)
+        {
+            return status;
+        }
         status = open_scanner (pss);
         if (status != SANE_STATUS_GOOD)
         {
@@ -968,7 +984,6 @@ SANE_Status sane_open (SANE_String_Const name, SANE_Handle * h)
 
         pss->state = ST_IDLE;
     }
-
     return SANE_STATUS_GOOD;
 }
 
@@ -986,6 +1001,7 @@ void sane_close (SANE_Handle h)
         break;
     }
     close_scanner (pss);
+    snapscani_usb_shm_exit();
     free (pss->gamma_tables);
     free (pss->buf);
     free (pss);
@@ -1245,7 +1261,7 @@ static SANE_Status send_gamma_table (SnapScan_Scanner *pss, u_char dtc, u_char d
     SANE_Status status = SANE_STATUS_GOOD;
     status = send (pss, dtc, dtcq);
     CHECK_STATUS (status, me, "send");
-    if (pss->pdev->model == PERFECTION1670)
+    if ((pss->pdev->model == PERFECTION1670) || (pss->pdev->model == PERFECTION2480))
     {
         /* Epson Perfection 1670 needs an extra invitation */
         status = send (pss, dtc, dtcq);
@@ -1551,7 +1567,7 @@ SANE_Status sane_start (SANE_Handle h)
 
     status = download_gamma_tables(pss);
     CHECK_STATUS (status, me, "download_gamma_tables");
-    if (pss->pdev->model == PERFECTION1670)
+    if ((pss->pdev->model == PERFECTION1670) || (pss->pdev->model == PERFECTION2480))
     {
         /* Epson Perfections needs the gamma table twice */
         status = download_gamma_tables(pss);
@@ -1821,6 +1837,23 @@ SANE_Status sane_get_select_fd (SANE_Handle h, SANE_Int * fd)
 
 /*
  * $Log: snapscan.c,v $
+ * Revision 1.45  2004/10/03 17:34:36  hmg-guest
+ * 64 bit platform fixes (bug #300799).
+ *
+ * Revision 1.44  2004/09/02 20:59:12  oliver-guest
+ * Added support for Epson 2480
+ *
+ * Revision 1.43  2004/06/16 19:52:26  oliver-guest
+ * Don't enforce even number of URB packages on 1212u_2. Fixes bug #300753.
+ *
+ * Revision 1.42  2004/06/15 12:17:37  hmg-guest
+ * Only use __attribute__ if gcc is used for compilation. Some other compilers
+ * don't know __attribute__ and therefore can't compile sane-backends without this
+ * fix. See bug #300803.
+ *
+ * Revision 1.41  2004/05/26 22:37:01  oliver-guest
+ * Use shared memory for urb counters in snapscan backend
+ *
  * Revision 1.40  2004/04/09 11:59:02  oliver-guest
  * Fixes for pthread implementation
  *
