@@ -42,7 +42,7 @@
 */
 
 /* 
-	$Id: teco2.h,v 1.8 2004/01/15 17:39:30 gekl-guest Exp $
+	$Id: teco2.h,v 1.9 2004/08/05 16:44:03 gekl-guest Exp $
 */
 
 /* Commands supported by the scanner. */
@@ -347,12 +347,6 @@ struct scanners_supported
 
 /*--------------------------------------------------------------------------*/
 
-#define BLACK_WHITE_STR		SANE_I18N("Black & White")
-#define GRAYSCALE_STR		SANE_I18N("Grayscale")
-#define COLOR_STR			SANE_I18N("Color")
-
-/*--------------------------------------------------------------------------*/
-
 /* Define a scanner occurence. */
 typedef struct Teco_Scanner
 {
