@@ -124,8 +124,8 @@ static void scanimage_exit (void);
 
 static void
 auth_callback (SANE_String_Const resource,
-	       SANE_Char username[SANE_MAX_USERNAME_LEN],
-	       SANE_Char password[SANE_MAX_PASSWORD_LEN])
+	       SANE_Char *username,
+	       SANE_Char *password)
 {
   char tmp[3 + 128 + SANE_MAX_USERNAME_LEN + SANE_MAX_PASSWORD_LEN], *wipe;
   unsigned char md5digest[16];
@@ -1983,7 +1983,16 @@ List of available devices:", prog_name);
                 {
                   fprintf (stderr, "Place document no. %d on the scanner.\n", n);
                   fprintf (stderr, "Press <RETURN> to continue.\n");
+                  fprintf (stderr, "Press Ctrl + D to terminate.\n");
                   readbuf2=fgets(readbuf, 2, stdin);
+
+		  if (readbuf2 == NULL)
+		    {
+		      fprintf (stderr, "Batch terminated, %d pages scanned\n", (n - batch_increment));
+		      fclose (stdout);
+		      unlink (path);
+		      break; /* get out of this loop */
+		    }
                 }
 	      fprintf (stderr, "Scanning page %d\n", n);
 	    }
