@@ -1,6 +1,6 @@
 /* sane-find-scanner.c
 
-   Copyright (C) 1997-2002 Oliver Rauch, Henning Meier-Geinitz, and others.
+   Copyright (C) 1997-2004 Oliver Rauch, Henning Meier-Geinitz, and others.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -30,13 +30,22 @@
 #include <dirent.h>
 #include <errno.h>
 
-#ifdef HAVE_LIBUSB
-#include "usb.h"
+#if defined (HAVE_WINDOWS_H)
+#include <windows.h>
+#include <ddk/scsi.h>
+#include <ddk/ntddscsi.h>
 #endif
 
 #include "../include/sane/sanei.h"
 #include "../include/sane/sanei_scsi.h"
 #include "../include/sane/sanei_usb.h"
+#include "../include/sane/sanei_pa4s2.h"
+#include "../include/sane/sanei_config.h"
+
+#ifdef HAVE_LIBUSB
+#include "usb.h"
+extern char * check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file);
+#endif
 
 #ifndef PATH_MAX
 # define PATH_MAX 1024
@@ -83,6 +92,9 @@ usage (char *msg)
   fprintf (stderr, "\t-q: be quiet (print only devices, no comments)\n");
   fprintf (stderr, "\t-f: force opening devname as SCSI even if it looks "
 	   "like USB\n");
+  fprintf (stderr, "\t-p: enable scannig for parallel port devices\n");
+  fprintf (stderr, "\t-F file: try to detect chipset from given "
+	   "/proc/bus/usb/devices file\n");
   if (msg)
     fprintf (stderr, "\t%s\n", msg);
 }
@@ -373,20 +385,45 @@ static char *
 get_libusb_string_descriptor (struct usb_device *dev, int index)
 {
   usb_dev_handle *handle;
-  char *buffer;
+  char *buffer, short_buffer[2];
   struct usb_string_descriptor *sd;
-  int size = 100;
+  int size = 2;
   int i;
 
   if (!index)
     return 0;
+
+
+  handle = usb_open (dev);
+  if (!handle)
+    return 0;
+
+  sd = (struct usb_string_descriptor *) short_buffer;
+
+  if (usb_control_msg (handle,
+		       USB_ENDPOINT_IN + USB_TYPE_STANDARD + USB_RECIP_DEVICE,
+		       USB_REQ_GET_DESCRIPTOR,
+		       (USB_DT_STRING << 8) + index, 0, short_buffer,
+		       size, 1000) < 0)
+    {
+      usb_close (handle);
+      return 0;
+    }
+
+  if (sd->bLength < 2 
+      || sd->bDescriptorType != USB_DT_STRING)
+    {
+      usb_close (handle);
+      return 0;
+    }
+  
+  size = sd->bLength;
+
   buffer = calloc (1, size + 1);
   if (!buffer)
     return 0;
   sd = (struct usb_string_descriptor *) buffer;
-  handle = usb_open (dev);
-  if (!handle)
-    return 0;
+
   if (usb_control_msg (handle,
 		       USB_ENDPOINT_IN + USB_TYPE_STANDARD + USB_RECIP_DEVICE,
 		       USB_REQ_GET_DESCRIPTOR,
@@ -396,6 +433,7 @@ get_libusb_string_descriptor (struct usb_device *dev, int index)
       usb_close (handle);
       return 0;
     }
+
   if (sd->bLength < 2 || sd->bLength > size
       || sd->bDescriptorType != USB_DT_STRING)
     {
@@ -424,7 +462,7 @@ get_libusb_product (struct usb_device *dev)
 }
 
 static void
-check_libusb_device (struct usb_device *dev)
+check_libusb_device (struct usb_device *dev, SANE_Bool from_file)
 {
   int is_scanner = 0;
   char *vendor = get_libusb_vendor (dev);
@@ -445,7 +483,6 @@ check_libusb_device (struct usb_device *dev)
       int config_nr;
       struct usb_device_descriptor *d = &dev->descriptor;
 
-
       printf ("\n");
       printf ("<device descriptor of 0x%04x/0x%04x at %s:%s",
 	      d->idVendor, d->idProduct, dev->bus->dirname, dev->filename);
@@ -509,7 +546,7 @@ check_libusb_device (struct usb_device *dev)
 		   alt_setting_nr++)
 		{
 		  struct usb_interface_descriptor *i
-		    = &interface->altsetting[0];
+		    = &interface->altsetting[alt_setting_nr];
 		  int ep_nr;
 		  printf ("   <altsetting %d>\n", alt_setting_nr);
 		  printf ("   bLength            %d\n", i->bLength);
@@ -602,13 +639,21 @@ check_libusb_device (struct usb_device *dev)
 
   if (is_scanner > 0)
     {
+      char * chipset = check_usb_chip (dev, verbose, from_file);
+
       printf ("found USB scanner (vendor=0x%04x", dev->descriptor.idVendor);
       if (vendor)
 	printf (" [%s]", vendor);
       printf (", product=0x%04x", dev->descriptor.idProduct);
       if (product)
 	printf (" [%s]", product);
-      printf (") at libusb:%s:%s\n", dev->bus->dirname, dev->filename);
+      if (chipset)
+	printf (", chip=%s", chipset);
+      if (from_file)
+	printf (")\n");
+      else
+	printf (") at libusb:%s:%s\n", dev->bus->dirname, dev->filename);
+
       libusb_device_found = SANE_TRUE;
       device_found = SANE_TRUE;
     }
@@ -671,10 +716,412 @@ get_next_file (char *dir_name, DIR * dir)
   return file_name;
 }
 
+#if defined (HAVE_WINDOWS_H)
+/* Return a list of potential scanners. There's a lot of hardcoded values here that might break on a system with lots of scsi devices. */
+static char **build_scsi_dev_list()
+{
+	char **dev_list;
+	int dev_list_index;
+	int hca;
+	HANDLE fd;
+	char scsi_hca_name[20];
+	char buffer[4096];
+	DWORD BytesReturned;
+	BOOL ret;
+	size_t dev_list_size;
+	PSCSI_ADAPTER_BUS_INFO adapter;
+	PSCSI_INQUIRY_DATA inquiry;
+	int i;
+
+	/* Allocate room for about 100 scanners. That should be enough. */
+	dev_list_size = 100;
+	dev_list_index = 0;
+	dev_list = calloc(1, dev_list_size * sizeof(char *));
+
+	hca = 0;
+
+	for(hca = 0; ; hca++) {
+
+		/* Open the adapter */
+		snprintf(scsi_hca_name, 20, "\\\\.\\Scsi%d:", hca);
+		fd = CreateFile(scsi_hca_name, GENERIC_READ | GENERIC_WRITE,
+						FILE_SHARE_READ | FILE_SHARE_WRITE,
+						NULL, OPEN_EXISTING,
+						FILE_FLAG_RANDOM_ACCESS, NULL );
+
+		if (fd == INVALID_HANDLE_VALUE) {
+			/* Assume there is no more adapter. This is wrong in the case
+			 * of hot-plug stuff, but I have yet to see it on a user
+			 * machine. */
+			break;
+		}
+
+		/* Get the inquiry info for the devices on that hca. */
+        ret = DeviceIoControl(fd,
+							  IOCTL_SCSI_GET_INQUIRY_DATA,
+							  NULL,
+							  0,
+							  buffer,
+							  sizeof(buffer),
+							  &BytesReturned,
+							  FALSE);
+
+        if(ret == 0)
+			{
+				CloseHandle(fd);
+				continue;
+			}
+
+		adapter = (PSCSI_ADAPTER_BUS_INFO)buffer;
+
+		for(i = 0; i < adapter->NumberOfBuses; i++) {	
+
+			if (adapter->BusData[i].InquiryDataOffset == 0) {
+				/* No device here */
+				continue;
+			}
+
+			inquiry = (PSCSI_INQUIRY_DATA) (buffer + 
+											adapter->BusData[i].InquiryDataOffset);
+			while(1) {
+				/* Check if it is a scanner or a processor
+				 * device. Ignore the other
+				 * device types. */
+				if (inquiry->InquiryDataLength >= 5 &&
+					((inquiry->InquiryData[0] & 0x1f) == 3 ||
+					 (inquiry->InquiryData[0] & 0x1f) == 6)) {
+					char device_name[20];
+					sprintf(device_name, "h%db%dt%dl%d", hca, inquiry->PathId, inquiry->TargetId, inquiry->Lun);
+					dev_list[dev_list_index] = strdup(device_name);
+					dev_list_index++;
+				}
+			
+				if (inquiry->NextInquiryDataOffset == 0) {
+					/* No device here */
+					break;
+				} else {
+					inquiry =  (PSCSI_INQUIRY_DATA) (buffer +
+													 inquiry->NextInquiryDataOffset);
+				}
+			}
+	    }
+
+		CloseHandle(fd);
+
+	}
+
+	return dev_list;
+
+}
+#endif
+
+#if defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
+char **scsi_dev_list;
+int scsi_dev_list_index;
+
+static SANE_Status AddToSCSIDeviceList (const char *dev) {
+  if (scsi_dev_list_index < 99) {
+    scsi_dev_list [scsi_dev_list_index] = strdup (dev);
+    scsi_dev_list_index++;
+    return SANE_STATUS_GOOD;
+  }
+  else
+    return SANE_STATUS_NO_MEM;
+}
+
+static char **build_scsi_dev_list()
+{
+  scsi_dev_list_index = 0;
+  scsi_dev_list = malloc (100 * sizeof(char *));
+  sanei_scsi_find_devices (NULL, NULL, NULL, -1, -1, -1, -1,
+			   AddToSCSIDeviceList);
+  scsi_dev_list [scsi_dev_list_index] = NULL;
+  return scsi_dev_list;
+}
+#endif
+
+static int
+check_mustek_pp_device (void)
+{
+  const char **devices;
+  int ctr = 0, found = 0, scsi = 0;
+
+  if (verbose > 1)
+    printf ("searching for Mustek parallel port scanners:\n");
+
+  devices = sanei_pa4s2_devices ();
+
+  while (devices[ctr] != NULL) {
+    int fd;
+    SANE_Status result;
+
+    /* ordinary parallel port scanner type */
+    if (verbose > 1)
+      printf ("checking %s...", devices[ctr]);
+
+    result = sanei_pa4s2_open (devices[ctr], &fd);
+    
+    if (verbose > 1)
+      {
+        if (result != 0)
+  	  printf (" failed to open (%s)\n", sane_strstatus (result));
+        else
+	  printf (" open ok\n");
+      }
+
+    if (result == 0) {
+      printf ("found possible Mustek parallel port scanner at \"%s\"\n",
+              devices[ctr]);
+      found++;
+      sanei_pa4s2_close(fd);
+    }
+    
+    /* trying scsi over pp devices */
+    if (verbose > 1)
+      printf ("checking %s (SCSI emulation)...", devices[ctr]);
+
+    result = sanei_pa4s2_scsi_pp_open (devices[ctr], &fd);
+    
+    if (verbose > 1)
+      {
+        if (result != 0)
+  	  printf (" failed to open (%s)\n", sane_strstatus (result));
+        else
+	  printf (" open ok\n");
+      }
+
+    if (result == 0) {
+      printf ("found possible Mustek SCSI over PP scanner at \"%s\"\n",
+              devices[ctr]);
+      scsi++;
+      sanei_pa4s2_close(fd);
+    }
+
+    ctr++;
+  }
+
+  free(devices);
+
+  if (found > 0 && verbose > 0)
+    printf("\n  # Your Mustek parallel port scanner was detected.  It may or\n"
+           "  # may not be supported by SANE.  Please read the sane-mustek_pp\n"
+	   "  # man-page for setup instructions.\n");
+
+  if (scsi > 0 && verbose > 0)
+    printf("\n  # Your Mustek parallel port scanner was detected.  It may or\n"
+           "  # may not be supported by SANE.  Please read the sane-mustek_pp\n"
+	   "  # man-page for setup instructions.\n");
+
+  return (found > 0 || scsi > 0);
+}
+
+#ifdef HAVE_LIBUSB
+static SANE_Bool
+parse_num (char* search, const char* line, int base, long int * number)
+{
+  char* start_number;
+
+  start_number = strstr (line, search);
+  if (start_number == NULL)
+    return SANE_FALSE;
+  start_number += strlen (search);
+ 
+  *number = strtol (start_number, NULL, base);
+  if (verbose > 2)
+    printf ("Found %s%ld\n", search, *number);
+  return SANE_TRUE;
+}
+
+static SANE_Bool
+parse_bcd (char* search, const char* line, long int * number)
+{
+  char* start_number;
+  char* end_number;
+  int first_part;
+  int second_part;
+
+  start_number = strstr (line, search);
+  if (start_number == NULL)
+    return SANE_FALSE;
+  start_number += strlen (search);
+ 
+  first_part = strtol (start_number, &end_number, 10);
+  start_number = end_number + 1; /* skip colon */
+  second_part = strtol (start_number, NULL, 10);
+  *number = ((first_part / 10) << 12) + ((first_part % 10) << 8) 
+    + ((second_part / 10) << 4) + (second_part % 10);
+  if (verbose > 2)
+    printf ("Found %s%ld\n", search, *number);
+  return SANE_TRUE;
+}
+
+static void
+parse_file (char *filename)
+{
+  FILE * parsefile;
+  char line [PATH_MAX], *token;
+  const char * p;
+  struct usb_device *dev = 0;
+  long int number = 0;
+  int current_config = 1;
+  int current_if = -1;
+  int current_as = -1;
+  int current_ep = -1;
+
+  if (verbose > 1)
+    printf ("trying to open %s\n", filename);
+  parsefile = fopen (filename, "r");
+
+  if (parsefile == NULL)
+    {
+      if (verbose > 0)
+	printf ("opening %s failed: %s\n", filename, strerror (errno));
+      return;
+    }
+
+  while (sanei_config_read (line, PATH_MAX, parsefile))
+    {
+      if (verbose > 2)
+	printf ("parsing line: `%s'\n", line);
+      p = sanei_config_get_string (line, &token);
+      if (!token || !p || token[0] == '\0')
+	continue;
+      if (token[1] != ':')
+	{
+	  if (verbose > 2)
+	    printf ("missing `:'?\n");
+	  continue;
+	}
+      switch (token[0])
+	{
+	case 'T':
+	  if (dev)
+	    check_libusb_device (dev, SANE_TRUE);
+	  dev = calloc (1, sizeof (struct usb_device));
+	  dev->bus = calloc (1, sizeof (struct usb_bus));
+	  current_config = 1;
+	  current_if = -1;
+	  current_as = -1;
+	  current_ep = -1;
+	  break;
+	case 'D':
+	  if (parse_bcd ("Ver=", line, &number))
+	    dev->descriptor.bcdUSB = number;
+	  if (parse_num ("Cls=", line, 16, &number))
+	    dev->descriptor.bDeviceClass = number;
+	  if (parse_num ("Sub=", line, 16, &number))
+	    dev->descriptor.bDeviceSubClass = number;
+	  if (parse_num ("Prot=", line, 16, &number))
+	    dev->descriptor.bDeviceProtocol = number;
+	  if (parse_num ("MxPS=", line, 10, &number))
+	    dev->descriptor.bMaxPacketSize0 = number;
+	  if (parse_num ("#Cfgs=", line, 10, &number))
+	    dev->descriptor.bNumConfigurations = number;
+	  dev->config = calloc (number, sizeof (struct usb_config_descriptor));
+	  break;
+	case 'P':
+	  if (parse_num ("Vendor=", line, 16, &number))
+	    dev->descriptor.idVendor = number;
+	  if (parse_num ("ProdID=", line, 16, &number))
+	    dev->descriptor.idProduct = number;
+	  if (parse_bcd ("Rev=", line, &number))
+	    dev->descriptor.bcdDevice = number;
+	  break;
+	case 'C':
+	  current_if = -1;
+	  current_as = -1;
+	  current_ep = -1;
+	  if (parse_num ("Cfg#=", line, 10, &number))
+	    {
+	      current_config = number - 1;
+	      dev->config[current_config].bConfigurationValue = number;
+	    }
+	  if (parse_num ("Ifs=", line, 10, &number))
+	    dev->config[current_config].bNumInterfaces = number;
+	  dev->config[current_config].interface 
+	    = calloc (number, sizeof (struct usb_interface));
+	  if (parse_num ("Atr=", line, 16, &number))
+	    dev->config[current_config].bmAttributes = number;
+	  if (parse_num ("MxPwr=", line, 10, &number))
+	    dev->config[current_config].MaxPower = number / 2;
+	  break;
+	case 'I':
+	  current_ep = -1;
+	  if (parse_num ("If#=", line, 10, &number))
+	    {
+	      if (current_if != number)
+		{
+		  current_if = number;
+		  current_as = -1;
+		  dev->config[current_config].interface[current_if].altsetting
+		    = calloc (20, sizeof (struct usb_interface_descriptor));
+		  /* Can't read number of altsettings */
+		  dev->config[current_config].interface[current_if].num_altsetting = 1;
+		}
+	      else
+		dev->config[current_config].interface[current_if].num_altsetting++;
+	    }
+	  if (parse_num ("Alt=", line, 10, &number))
+	    {
+	      current_as = number;
+	      dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceNumber 
+		= current_if;
+	      dev->config[current_config].interface[current_if].altsetting[current_as].bAlternateSetting 
+		= current_as;
+	    }
+	  if (parse_num ("#EPs=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bNumEndpoints 
+	      = number;
+	  dev->config[current_config].interface[current_if].altsetting[current_as].endpoint 
+	    = calloc (number, sizeof (struct usb_endpoint_descriptor));
+	  if (parse_num ("Cls=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceClass 
+	      = number;
+	  if (parse_num ("Sub=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceSubClass 
+	      = number;
+	  if (parse_num ("Prot=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as].bInterfaceProtocol 
+	      = number;
+	  break;
+	case 'E':
+	  current_ep++;
+	  if (parse_num ("Ad=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bEndpointAddress = number;
+	  if (parse_num ("Atr=", line, 16, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bmAttributes = number;
+	  if (parse_num ("MxPS=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].wMaxPacketSize = number;
+	  if (parse_num ("Ivl=", line, 10, &number))
+	    dev->config[current_config].interface[current_if].altsetting[current_as]
+	      .endpoint[current_ep].bInterval = number;
+	  break;
+	case 'S':
+	case 'B':
+	  continue;
+	default:
+	  if (verbose > 1)
+	    printf ("ignoring unknown line identifier: %c\n", token[0]);
+	  continue;
+	}
+      free (token);
+    }
+  if (dev)
+    check_libusb_device (dev, SANE_TRUE);
+  fclose (parsefile);
+  return;
+}
+#endif
+
 int
 main (int argc, char **argv)
 {
   char **dev_list, **usb_dev_list, *dev_name, **ap;
+  int enable_pp_checks = SANE_FALSE;
 
   prog_name = strrchr (argv[0], '/');
   if (prog_name)
@@ -705,6 +1152,18 @@ main (int argc, char **argv)
 	  force = SANE_TRUE;
 	  break;
 
+	case 'p':
+	  enable_pp_checks = SANE_TRUE;
+	  break;
+
+	case 'F':
+#ifdef HAVE_LIBUSB
+	  parse_file ((char *) (*(++ap)));
+#else
+	  printf ("libusb not available: option -F can't be used\n");
+#endif
+	  exit (0);
+
 	default:
 	  printf ("unknown option: -%c, try -h for help\n", (*ap)[1]);
 	  exit (0);
@@ -942,6 +1401,16 @@ main (int argc, char **argv)
 	"/dev/uscanner10", "/dev/uscanner11",
 	"/dev/uscanner12", "/dev/uscanner13",
 	"/dev/uscanner14", "/dev/uscanner15",
+#elif defined(__NetBSD__)
+	"/dev/uscanner",
+	"/dev/uscanner0", "/dev/uscanner1",
+	"/dev/uscanner2", "/dev/uscanner3",
+	"/dev/uscanner4", "/dev/uscanner5",
+	"/dev/uscanner6", "/dev/uscanner7",
+	"/dev/uscanner8", "/dev/uscanner9",
+	"/dev/uscanner10", "/dev/uscanner11",
+	"/dev/uscanner12", "/dev/uscanner13",
+	"/dev/uscanner14", "/dev/uscanner15",
 #elif defined(__OpenBSD__)
 	"/dev/uscanner",
 	"/dev/uscanner0", "/dev/uscanner1",
@@ -956,10 +1425,19 @@ main (int argc, char **argv)
 	0
       };
 
+#if defined (HAVE_WINDOWS_H) || \
+    defined (HAVE_IOKIT_CDB_IOSCSILIB_H) || \
+    defined (HAVE_IOKIT_SCSI_COMMANDS_SCSICOMMANDOPERATIONCODES_H)
+   /* Build a list of valid of possible scanners found */
+      dev_list = build_scsi_dev_list();
+#else
       dev_list = default_dev_list;
+#endif
+
       usb_dev_list = usb_default_dev_list;
     }
-
+  if (verbose > 1)
+    printf ("This is sane-find-scanner from %s\n", PACKAGE_STRING);
   if (verbose > 0)
     printf ("\n");
   if (verbose > 1)
@@ -1064,11 +1542,14 @@ main (int argc, char **argv)
 	  {
 	    for (dev = bus->devices; dev; dev = dev->next)
 	      {
-		check_libusb_device (dev);
+		check_libusb_device (dev, SANE_FALSE);
 	      }			/* for (dev) */
 	  }			/* for (bus) */
       }				/* if (usb_dev_list == ap) */
   }
+#else
+  if (verbose > 1)
+    printf ("libusb not available\n");
 #endif /* HAVE_LIBUSB */
 
   if (device_found)
@@ -1102,9 +1583,19 @@ main (int argc, char **argv)
 	   "make sure that\n  # you have loaded a driver for your USB host "
 	   "controller and have installed a\n  # kernel scanner module.\n");
     }
+  if (enable_pp_checks == SANE_TRUE) 
+    {
+      if (!check_mustek_pp_device() && verbose > 0)
+        printf ("\n  # No Mustek parallel port scanners found. If you expected"
+                " something\n  # different, make sure the scanner is correctly"
+	        " connected to your computer\n  # and you have apropriate"
+	        " access rights.\n");
+    }
+  else if (verbose > 0)
+    printf ("\n  # Not checking for parallel port scanners.\n");
   if (verbose > 0)
-    printf ("\n  # Scanners connected to the parallel port or other "
-	    "proprietary ports can't be\n  # detected by this program.\n");
+    printf ("\n  # Most Scanners connected to the parallel port or other "
+	    "proprietary ports\n  # can't be detected by this program.\n");
   if (getuid ())
     if (verbose > 0)
       printf
